package com.persagy.apm.energy.report.monthly.detail.hotel.project.service.impl;

import com.persagy.apm.energy.report.monthly.config.function.constant.enums.FunctionTypeEnum;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.model.dto.QueryFunctionDTO;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.service.impl.ReportCostInfoServiceImpl;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectCostInfo;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportCostService;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 酒店项目类报告费用service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public class HotelProjectReportCostServiceImpl extends ReportCostInfoServiceImpl
        implements IHotelProjectReportCostService {
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private ISaasWebService saasWebService;
    @Autowired
    private IReportOutlineService reportOutlineService;

    @Override
    public HotelProjectCostInfo getCost(ReportOutline reportOutline, List<String> functionIdList) {
        Date reportMonth = reportOutline.getReportMonth();
        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);
        if (CollectionUtils.isEmpty(projectIds)) {
            return null;
        }
        return getCostInfoByProjectId(functionIdList, reportMonth, projectIds.get(0));
    }

    @Override
    public HotelProjectCostInfo getCostInfoByProjectId(
            List<String> functionIdList, Date reportMonth, String projectId) {
        HotelProjectCostInfo hotelProjectCostInfo = new HotelProjectCostInfo();
        // 获取项目信息
        SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
        if (simpleProjectInfo == null) {
            return null;
        }
        hotelProjectCostInfo.setProjectName(simpleProjectInfo.getProjectName());

        List<CostItemVO> costItemInfos = getCostItemInfos(functionIdList, reportMonth, Lists.newArrayList(projectId));
        hotelProjectCostInfo.setItemsCostInfo(costItemInfos);

        // 获取信息点类型为费用的信息点, 计算项目总的费用信息
        QueryFunctionDTO queryFunctionDTO = new QueryFunctionDTO();
        queryFunctionDTO.setFunctionIdList(functionIdList);
        queryFunctionDTO.setFunctionType(FunctionTypeEnum.COST.getType());
        List<Function> costFunctions = functionService.queryFunctionList(queryFunctionDTO);

        if (!CollectionUtils.isEmpty(costFunctions)) {
            List<String> costFunctionIds = costFunctions.stream().map(Function::getId).collect(Collectors.toList());
            List<CostItemVO> costFunctionsInfo = getCostItemInfos(costFunctionIds, reportMonth, Lists.newArrayList(projectId));
            // 总费用相关信息
            CostItemVO costSum = getCostSum(costFunctionsInfo);
            hotelProjectCostInfo.setTotalCost(costSum.getCurrentMonth());
            hotelProjectCostInfo.setCostSameTimeRange(costSum.getSameTimeRange());
            hotelProjectCostInfo.setCostPerSquareMetre(costSum.getYearPerCount());
        }

        return hotelProjectCostInfo;
    }

    @Override
    public Double getProjectBuildingArea(List<String> projectIds) {
        return saasWebService.getTotalBuildingArea(projectIds);
    }
}
